;;; parse-time-rfc2822.el --- Parse a RFC2822 compliant date format.

;; Copyright (C) 2013  Andreas Politz

;; Author: Andreas Politz <politza@hochschule-trier.de>
;; Keywords: mail

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;; 

;;; Code:

(require 'parse-time)
(require 'cl-lib)

(defconst parse-time-rfc2822-regex
  (let* ((fws "[ \t\r\n]+")
         (ofws "[ \t\r\n]*")
         (day-of-week
          (regexp-opt
           (mapcar 'car parse-time-weekdays)))
         (day "[0-9]\\{1,2\\}")
         (month
          (regexp-opt
           (mapcar 'car parse-time-months)))
         (year "\\(?:[0-9]\\{2\\}\\|[0-9]\\{4,\\}\\)")
         (hour "[0-9]\\{2\\}")
         (minute hour)
         (second hour)
         (zone "[+-][0-9]\\{4\\}")
         (zone-text
          (regexp-opt (mapcar 'car parse-time-zoneinfo)))
         ;; A rather non strict comment.
         (cfws "(\\(?:.\\|\n\\)*)"))
    (concat
     (format "\\`%s\\(?:\\(%s\\),%s\\)?" ofws day-of-week ofws)
     (format "\\(%s\\)%s\\(%s\\)%s\\(%s\\)%s"
             day fws month fws year fws)
     (format "\\(%s\\):\\(%s\\)\\(?::\\(%s\\)\\)?%s"
             hour minute second fws)
     (format "\\(?:\\(%s\\)\\|\\(%s\\)\\)" zone zone-text)
     (format "%s\\(?:%s\\)?%s\\'" ofws cfws ofws)))
  "A regex matching a strict rfc2822 date.")

(defun parse-time-string-rfc2822 (string)
  "Parse the strict rfc2822 time-string STRING

Return either a list (SEC MIN HOUR DAY MON YEAR DOW DST TZ) or
nil, if STRING is not valid."
  (setq s string)
  (when (string-match parse-time-rfc2822-regex
                      (setq string (downcase string)))
    (let ((dow (cdr (assoc (match-string 1 string)
                           parse-time-weekdays)))
          (day (string-to-number (match-string 2 string)))
          (month (cdr (assoc (match-string 3 string)
                             parse-time-months)))
          (year (string-to-number (match-string 4 string)))
          (hour (string-to-number (match-string 5 string)))
          (min (string-to-number (match-string 6 string)))
          (sec (string-to-number (or (match-string 7 string) "0")))
          zone dst)
      (if (match-string 9 string)
          (let ((zoneinfo (cdr (assoc (match-string 9 string)
                                      parse-time-zoneinfo))))
            (setq dst (cadr zoneinfo)
                  zone (car zoneinfo)))
        (setq zone (string-to-number (match-string 8 string))))
      ;; adjust 2 digit RFC822 years (1969 - 2068)
      (when (< year 100)
        (if (> year 68)
            (cl-incf year 1900)
          (cl-incf year 2000)))
      (let ((zone (+ (* (/ zone 100) 3600)
                     (* (% zone 100) 60))))
        (list sec min hour day month year dow dst zone)))))

(provide 'parse-time-rfc2822)
;;; parse-time-rfc2822.el ends here
